#!/usr/bin/env perl

use 5.010001;
use utf8;
# use utf8::all;
use strict;
use warnings;
use warnings FATAL => 'utf8';
use autodie;

use open qw[ :utf8 :std ];
use Mojo::DOM;
use URI::Escape qw[uri_escape];
use MIME::Base64 qw[encode_base64];
use Path::Tiny qw[path];

# The Mojo::DOM::CSS selector for the elements to modify
my $selector = <<'SELECTOR';
img,
a.base64,
a[data-base64]
SELECTOR

# Get the HTML from stdin as a string
my $html = do { local $/; <>; };

# Create a DOM tree object representing the HTML 
my $dom  = Mojo::DOM->new($html);

# Get a collection of the elements to modify
my $elems = $dom->find($selector);

# Modify the elements in place
ELEM:
for my $elem ( @$elems ) {
  # Get the URL attribute name; why not `url` in both?! :-(
  my($url) = grep { exists $elem->{$_} } qw(src href);

  # Unless there is a `data-base64` attribute...
  unless ( $elem->{'data-base64'} ) {
    # ... skip this element if it already has a data url
    next ELEM if ($elem->{$url} // "") =~ /^data:/;
  }

  # Get the path to the local image file from eithet
  # the `data-base64` attribute
  # or the URL attribute
  # or skip this element if neither attribute is defined
  my $img_path = path $elem->{'data-base64'} // $elem->{$url} // next ELEM;

  # Check that the file actually exists or issue warning and skip this element
  unless ( $img_path->is_file ) {
    warn "WARNING: '$img_path' isn't a local file!\n";
    next ELEM;
  }

  # Fetch and base-64 encode the image data
  my $data = uri_escape encode_base64 $img_path->slurp_raw;

  # Get the MIME type from the `data-mime` attribute
  # or fall back to "image/jpeg"
  my $mime = $elem->{'data-mime'} // 'image/jpeg';

  # Insert the data URL
  $elem->{$url} = "data:$mime;base64,$data";

  # Remove our custom attributes
  delete $elem->{"data-$_"} for qw(base64 mime);

  # Remove our custom class
  if ( my $classes = $elem->{class} ) {  # got any classes?
    if ( 'base64' eq $classes ) { # got just this one class?
      delete $elem->{class};
    }
    else {
      $classes =~ s{\bbase64\b}{}g; # remove our class
      # Fix whitespace
      $classes =~ s{\s+}{ }g; # collapse
      $classes =~ s{^\s+}{};  # remove leading
      $classes =~ s{\s+$}{};  # remove trailing
      # Update the class attribute
      $elem->{class} = $classes;
      delete $elem->{class} unless $classes
    }
  }
}

# Print the modified HTML to stdout
print $dom;


# This software is copyright (c) 2022 by Benct Philip Jonsson.

# This is free software; you can redistribute it and/or modify it under
# the same terms as the Perl 5 programming language system itself.

# http://dev.perl.org/licenses/

